@@ -1,37 +1,43 @@
 # pylint: disable=too-many-lines
 """Nginx Configuration"""
-from distutils.version import LooseVersion
 import logging
 import re
 import socket
 import subprocess
 import tempfile
 import time
+from typing import Any
+from typing import Callable
 from typing import Dict
+from typing import Iterable
 from typing import List
+from typing import Mapping
 from typing import Optional
+from typing import Sequence
 from typing import Set
 from typing import Text
 from typing import Tuple
+from typing import Type
+from typing import Union
 
+from certbot_nginx._internal import constants
+from certbot_nginx._internal import display_ops
+from certbot_nginx._internal import http_01
+from certbot_nginx._internal import nginxparser
+from certbot_nginx._internal import obj
+from certbot_nginx._internal import parser
 import OpenSSL
 import pkg_resources
 
 from acme import challenges
 from acme import crypto_util as acme_crypto_util
+from certbot import achallenges
 from certbot import crypto_util
 from certbot import errors
-from certbot import interfaces
 from certbot import util
-from certbot.display import util as display_util
 from certbot.compat import os
+from certbot.display import util as display_util
 from certbot.plugins import common
-from certbot_nginx._internal import constants
-from certbot_nginx._internal import display_ops
-from certbot_nginx._internal import http_01
-from certbot_nginx._internal import nginxparser
-from certbot_nginx._internal import obj  # pylint: disable=unused-import
-from certbot_nginx._internal import parser
 
 NAME_RANK = 0
 START_WILDCARD_RANK = 1
@@ -43,7 +49,7 @@ NO_SSL_MODIFIER = 4
 logger = logging.getLogger(__name__)
 
 
-class NginxConfigurator(common.Installer, interfaces.Authenticator):
+class NginxConfigurator(common.Configurator):
     """Nginx configurator.
 
     .. todo:: Add proper support for comments in the config. Currently,
@@ -72,7 +78,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
     SSL_DIRECTIVES = ['ssl_certificate', 'ssl_certificate_key', 'ssl_dhparam']
 
     @classmethod
-    def add_parser_arguments(cls, add):
+    def add_parser_arguments(cls, add: Callable[..., None]) -> None:
         default_server_root = _determine_default_server_root()
         add("server-root", default=constants.CLI_DEFAULTS["server_root"],
             help="Nginx server root directory. (default: %s)" % default_server_root)
@@ -84,11 +90,11 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             "to apply when reloading.")
 
     @property
-    def nginx_conf(self):
+    def nginx_conf(self) -> str:
         """Nginx config file path."""
         return os.path.join(self.conf("server_root"), "nginx.conf")
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Initialize an Nginx Configurator.
 
         :param tup version: version of Nginx as a tuple (1, 4, 7)
@@ -127,7 +133,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         self.parser: parser.NginxParser
 
     @property
-    def mod_ssl_conf_src(self):
+    def mod_ssl_conf_src(self) -> str:
         """Full absolute path to SSL configuration file source."""
 
         # Why all this complexity? Well, we want to support Mozilla's intermediate
@@ -142,8 +148,9 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         # For a complete history, check out https://github.com/certbot/certbot/issues/7322
 
         use_tls13 = self.version >= (1, 13, 0)
+        min_openssl_version = util.parse_loose_version('1.0.2l')
         session_tix_off = self.version >= (1, 5, 9) and self.openssl_version and\
-            LooseVersion(self.openssl_version) >= LooseVersion('1.0.2l')
+            util.parse_loose_version(self.openssl_version) >= min_openssl_version
 
         if use_tls13:
             if session_tix_off:
@@ -160,22 +167,23 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             "certbot_nginx", os.path.join("_internal", "tls_configs", config_filename))
 
     @property
-    def mod_ssl_conf(self):
+    def mod_ssl_conf(self) -> str:
         """Full absolute path to SSL configuration file."""
         return os.path.join(self.config.config_dir, constants.MOD_SSL_CONF_DEST)
 
     @property
-    def updated_mod_ssl_conf_digest(self):
+    def updated_mod_ssl_conf_digest(self) -> str:
         """Full absolute path to digest of updated SSL configuration file."""
         return os.path.join(self.config.config_dir, constants.UPDATED_MOD_SSL_CONF_DIGEST)
 
-    def install_ssl_options_conf(self, options_ssl, options_ssl_digest):
+    def install_ssl_options_conf(self, options_ssl: str, options_ssl_digest: str) -> None:
         """Copy Certbot's SSL options file into the system's config dir if required."""
-        return common.install_version_controlled_file(options_ssl, options_ssl_digest,
+        common.install_version_controlled_file(
+            options_ssl, options_ssl_digest,
             self.mod_ssl_conf_src, constants.ALL_SSL_OPTIONS_HASHES)
 
     # This is called in determine_authenticator and determine_installer
-    def prepare(self):
+    def prepare(self) -> None:
         """Prepare the authenticator/installer.
 
         :raises .errors.NoInstallationError: If Nginx ctl cannot be found
@@ -211,8 +219,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             raise errors.PluginError('Unable to lock {0}'.format(self.conf('server-root')))
 
     # Entry point in main.py for installing cert
-    def deploy_cert(self, domain, cert_path, key_path,
-                    chain_path=None, fullchain_path=None):
+    def deploy_cert(self, domain: str, cert_path: str, key_path: str, chain_path: str,
+                    fullchain_path: str) -> None:
         """Deploys certificate to specified virtual host.
 
         .. note:: Aborts if the vhost is missing ssl_certificate or
@@ -235,7 +243,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             display_util.notify("Successfully deployed certificate for {} to {}"
                                 .format(domain, vhost.filep))
 
-    def _deploy_cert(self, vhost, cert_path, key_path, chain_path, fullchain_path):  # pylint: disable=unused-argument
+    def _deploy_cert(self, vhost: obj.VirtualHost, _cert_path: str, key_path: str,
+                     _chain_path: str, fullchain_path: str) -> None:
         """
         Helper function for deploy_cert() that handles the actual deployment
         this exists because we might want to do multiple deployments per
@@ -245,8 +254,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         cert_directives = [['\n    ', 'ssl_certificate', ' ', fullchain_path],
                            ['\n    ', 'ssl_certificate_key', ' ', key_path]]
 
-        self.parser.update_or_add_server_directives(vhost,
-                                          cert_directives)
+        self.parser.update_or_add_server_directives(vhost, cert_directives)
         logger.info("Deploying Certificate to VirtualHost %s", vhost.filep)
 
         self.save_notes += ("Changed vhost at %s with addresses of %s\n" %
@@ -255,14 +263,17 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         self.save_notes += "\tssl_certificate %s\n" % fullchain_path
         self.save_notes += "\tssl_certificate_key %s\n" % key_path
 
-    def _choose_vhosts_wildcard(self, domain, prefer_ssl, no_ssl_filter_port=None):
+    def _choose_vhosts_wildcard(self, domain: str, prefer_ssl: bool,
+                                no_ssl_filter_port: Optional[str] = None) -> List[obj.VirtualHost]:
         """Prompts user to choose vhosts to install a wildcard certificate for"""
         if prefer_ssl:
             vhosts_cache = self._wildcard_vhosts
-            preference_test = lambda x: x.ssl
+            def preference_test(x: obj.VirtualHost) -> bool:
+                return x.ssl
         else:
             vhosts_cache = self._wildcard_redirect_vhosts
-            preference_test = lambda x: not x.ssl
+            def preference_test(x: obj.VirtualHost) -> bool:
+                return not x.ssl
 
         # Caching!
         if domain in vhosts_cache:
@@ -304,12 +315,13 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
     #######################
     # Vhost parsing methods
     #######################
-    def _choose_vhost_single(self, target_name):
+    def _choose_vhost_single(self, target_name: str) -> List[obj.VirtualHost]:
         matches = self._get_ranked_matches(target_name)
         vhosts = [x for x in [self._select_best_name_match(matches)] if x is not None]
         return vhosts
 
-    def choose_vhosts(self, target_name, create_if_no_match=False):
+    def choose_vhosts(self, target_name: str,
+                      create_if_no_match: bool = False) -> List[obj.VirtualHost]:
         """Chooses a virtual host based on the given domain name.
 
         .. note:: This makes the vhost SSL-enabled if it isn't already. Follows
@@ -353,7 +365,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         return vhosts
 
-    def ipv6_info(self, port):
+    def ipv6_info(self, port: str) -> Tuple[bool, bool]:
         """Returns tuple of booleans (ipv6_active, ipv6only_present)
         ipv6_active is true if any server block listens ipv6 address in any port
 
@@ -366,9 +378,6 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             configuration, and existence of ipv6only directive for specified port
         :rtype: tuple of type (bool, bool)
         """
-        # port should be a string, but it's easy to mess up, so let's
-        # make sure it is one
-        port = str(port)
         vhosts = self.parser.get_vhosts()
         ipv6_active = False
         ipv6only_present = False
@@ -378,10 +387,10 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
                     ipv6_active = True
                 if addr.ipv6only and addr.get_port() == port:
                     ipv6only_present = True
-        return (ipv6_active, ipv6only_present)
+        return ipv6_active, ipv6only_present
 
-    def _vhost_from_duplicated_default(self, domain: str, allow_port_mismatch: bool, port: str
-                                       ) -> obj.VirtualHost:
+    def _vhost_from_duplicated_default(self, domain: str, allow_port_mismatch: bool,
+                                       port: str) -> obj.VirtualHost:
         """if allow_port_mismatch is False, only server blocks with matching ports will be
            used as a default server block template.
         """
@@ -396,7 +405,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         self._add_server_name_to_vhost(self.new_vhost, domain)
         return self.new_vhost
 
-    def _add_server_name_to_vhost(self, vhost, domain):
+    def _add_server_name_to_vhost(self, vhost: obj.VirtualHost, domain: str) -> None:
         vhost.names.add(domain)
         name_block = [['\n    ', 'server_name']]
         for name in vhost.names:
@@ -404,7 +413,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             name_block[0].append(name)
         self.parser.update_or_add_server_directives(vhost, name_block)
 
-    def _get_default_vhost(self, domain, allow_port_mismatch, port):
+    def _get_default_vhost(self, domain: str, allow_port_mismatch: bool,
+                           port: str) -> obj.VirtualHost:
         """Helper method for _vhost_from_duplicated_default; see argument documentation there"""
         vhost_list = self.parser.get_vhosts()
         # if one has default_server set, return that one
@@ -425,10 +435,11 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         # TODO: present a list of vhosts for user to choose from
 
-        raise errors.MisconfigurationError("Could not automatically find a matching server"
-            " block for %s. Set the `server_name` directive to use the Nginx installer." % domain)
+        raise errors.MisconfigurationError("Could not automatically find a matching server "
+                                           f"block for {domain}. Set the `server_name` directive "
+                                           "to use the Nginx installer.")
 
-    def _get_ranked_matches(self, target_name):
+    def _get_ranked_matches(self, target_name: str) -> List[Dict[str, Any]]:
         """Returns a ranked list of vhosts that match target_name.
         The ranking gives preference to SSL vhosts.
 
@@ -441,7 +452,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         vhost_list = self.parser.get_vhosts()
         return self._rank_matches_by_name_and_ssl(vhost_list, target_name)
 
-    def _select_best_name_match(self, matches):
+    def _select_best_name_match(self,
+                                matches: Sequence[Mapping[str, Any]]) -> Optional[obj.VirtualHost]:
         """Returns the best name match of a ranked list of vhosts.
 
         :param list matches: list of dicts containing the vhost, the matching name,
@@ -461,7 +473,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         # Exact or regex match
         return matches[0]['vhost']
 
-    def _rank_matches_by_name(self, vhost_list, target_name):
+    def _rank_matches_by_name(self, vhost_list: Iterable[obj.VirtualHost],
+                              target_name: str) -> List[Dict[str, Any]]:
         """Returns a ranked list of vhosts from vhost_list that match target_name.
         This method should always be followed by a call to _select_best_name_match.
 
@@ -498,7 +511,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
                                 'rank': REGEX_RANK})
         return sorted(matches, key=lambda x: x['rank'])
 
-    def _rank_matches_by_name_and_ssl(self, vhost_list, target_name):
+    def _rank_matches_by_name_and_ssl(self, vhost_list: Iterable[obj.VirtualHost],
+                                      target_name: str) -> List[Dict[str, Any]]:
         """Returns a ranked list of vhosts from vhost_list that match target_name.
         The ranking gives preference to SSLishness before name match level.
 
@@ -597,8 +611,9 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         # if we want ssl vhosts: either 'ssl on' or 'addr.ssl' should be enabled
         # if we want plaintext vhosts: neither 'ssl on' nor 'addr.ssl' should be enabled
-        _ssl_matches = lambda addr: addr.ssl or all_addrs_are_ssl if ssl else \
-                                    not addr.ssl and not all_addrs_are_ssl
+        def _ssl_matches(addr: obj.Addr) -> bool:
+            return addr.ssl or all_addrs_are_ssl if ssl else \
+                   not addr.ssl and not all_addrs_are_ssl
 
         # if there are no listen directives at all, Nginx defaults to
         # listening on port 80.
@@ -611,7 +626,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
     def _vhost_listening_on_port_no_ssl(self, vhost: obj.VirtualHost, port: str) -> bool:
         return self._vhost_listening(vhost, port, False)
 
-    def _get_redirect_ranked_matches(self, target_name, port):
+    def _get_redirect_ranked_matches(self, target_name: str, port: str) -> List[Dict[str, Any]]:
         """Gets a ranked list of plaintextish port-listening vhosts matching target_name
 
         Filter all hosts for those listening on port without using ssl.
@@ -626,14 +641,14 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         """
         all_vhosts = self.parser.get_vhosts()
 
-        def _vhost_matches(vhost, port):
+        def _vhost_matches(vhost: obj.VirtualHost, port: str) -> bool:
             return self._vhost_listening_on_port_no_ssl(vhost, port)
 
         matching_vhosts = [vhost for vhost in all_vhosts if _vhost_matches(vhost, port)]
 
         return self._rank_matches_by_name(matching_vhosts, target_name)
 
-    def get_all_names(self):
+    def get_all_names(self) -> Set[str]:
         """Returns all names found in the Nginx Configuration.
 
         :returns: All ServerNames, ServerAliases, and reverse DNS entries for
@@ -671,7 +686,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         return util.get_filtered_names(all_names)
 
-    def _get_snakeoil_paths(self):
+    def _get_snakeoil_paths(self) -> Tuple[str, str]:
         """Generate invalid certs that let us create ssl directives for Nginx"""
         # TODO: generate only once
         tmp_dir = os.path.join(self.config.work_dir, "snakeoil")
@@ -689,7 +704,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             cert_file.write(cert_pem)
         return cert_path, le_key.file
 
-    def _make_server_ssl(self, vhost):
+    def _make_server_ssl(self, vhost: obj.VirtualHost) -> None:
         """Make a server SSL.
 
         Make a server SSL by adding new listen and SSL directives.
@@ -699,7 +714,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         """
         https_port = self.config.https_port
-        ipv6info = self.ipv6_info(https_port)
+        ipv6info = self.ipv6_info(str(https_port))
         ipv6_block = ['']
         ipv4_block = ['']
 
@@ -746,11 +761,12 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
     ##################################
     # enhancement methods (Installer)
     ##################################
-    def supported_enhancements(self):
+    def supported_enhancements(self) -> List[str]:
         """Returns currently supported enhancements."""
         return ['redirect', 'ensure-http-header', 'staple-ocsp']
 
-    def enhance(self, domain, enhancement, options=None):
+    def enhance(self, domain: str, enhancement: str,
+                options: Optional[Union[str, List[str]]] = None) -> None:
         """Enhance configuration.
 
         :param str domain: domain to enhance
@@ -762,16 +778,16 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         """
         try:
-            return self._enhance_func[enhancement](domain, options)
+            self._enhance_func[enhancement](domain, options)
         except (KeyError, ValueError):
             raise errors.PluginError(
                 "Unsupported enhancement: {0}".format(enhancement))
 
-    def _has_certbot_redirect(self, vhost, domain):
+    def _has_certbot_redirect(self, vhost: obj.VirtualHost, domain: str) -> bool:
         test_redirect_block = _test_block_from_block(_redirect_block_for_domain(domain))
         return vhost.contains_list(test_redirect_block)
 
-    def _set_http_header(self, domain, header_substring):
+    def _set_http_header(self, domain: str, header_substring: Union[str, List[str], None]) -> None:
         """Enables header identified by header_substring on domain.
 
         If the vhost is listening plaintextishly, separates out the relevant
@@ -785,7 +801,10 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         :raises .errors.PluginError: If no viable HTTPS host can be created or
             set with header header_substring.
         """
-        if not header_substring in constants.HEADER_ARGS:
+        if not isinstance(header_substring, str):
+            raise errors.NotSupportedError("Invalid header_substring type "
+                                           f"{type(header_substring)}, expected a str.")
+        if header_substring not in constants.HEADER_ARGS:
             raise errors.NotSupportedError(
                 f"{header_substring} is not supported by the nginx plugin.")
 
@@ -809,7 +828,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
                 ['\n']]
             self.parser.add_server_directives(vhost, header_directives)
 
-    def _add_redirect_block(self, vhost, domain):
+    def _add_redirect_block(self, vhost: obj.VirtualHost, domain: str) -> None:
         """Add redirect directive to vhost
         """
         redirect_block = _redirect_block_for_domain(domain)
@@ -817,7 +836,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         self.parser.add_server_directives(
             vhost, redirect_block, insert_at_top=True)
 
-    def _split_block(self, vhost, only_directives=None):
+    def _split_block(self, vhost: obj.VirtualHost, only_directives: Optional[List[str]] = None
+                     ) -> Tuple[obj.VirtualHost, obj.VirtualHost]:
         """Splits this "virtual host" (i.e. this nginx server block) into
         separate HTTP and HTTPS blocks.
 
@@ -830,13 +850,13 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         """
         http_vhost = self.parser.duplicate_vhost(vhost, only_directives=only_directives)
 
-        def _ssl_match_func(directive):
+        def _ssl_match_func(directive: str) -> bool:
             return 'ssl' in directive
 
-        def _ssl_config_match_func(directive):
+        def _ssl_config_match_func(directive: str) -> bool:
             return self.mod_ssl_conf in directive
 
-        def _no_ssl_match_func(directive):
+        def _no_ssl_match_func(directive: str) -> bool:
             return 'ssl' not in directive
 
         # remove all ssl addresses and related directives from the new block
@@ -850,7 +870,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         self.parser.remove_server_directives(vhost, 'listen', match_func=_no_ssl_match_func)
         return http_vhost, vhost
 
-    def _enable_redirect(self, domain, unused_options):
+    def _enable_redirect(self, domain: str,
+                         unused_options: Optional[Union[str, List[str]]]) -> None:
         """Redirect all equivalent HTTP traffic to ssl_vhost.
 
         If the vhost is listening plaintextishly, separate out the
@@ -877,7 +898,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         for vhost in vhosts:
             self._enable_redirect_single(domain, vhost)
 
-    def _enable_redirect_single(self, domain, vhost):
+    def _enable_redirect_single(self, domain: str, vhost: obj.VirtualHost) -> None:
         """Redirect all equivalent HTTP traffic to ssl_vhost.
 
         If the vhost is listening plaintextishly, separate out the
@@ -906,7 +927,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             logger.info("Redirecting all traffic on port %s to ssl in %s",
                 self.DEFAULT_LISTEN_PORT, vhost.filep)
 
-    def _enable_ocsp_stapling(self, domain, chain_path):
+    def _enable_ocsp_stapling(self, domain: str,
+                              chain_path: Optional[Union[str, List[str]]]) -> None:
         """Include OCSP response in TLS handshake
 
         :param str domain: domain to enable OCSP response for
@@ -914,11 +936,15 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         :type chain_path: `str` or `None`
 
         """
+        if not isinstance(chain_path, str) and chain_path is not None:
+            raise errors.NotSupportedError(f"Invalid chain_path type {type(chain_path)}, "
+                                           "expected a str or None.")
         vhosts = self.choose_vhosts(domain)
         for vhost in vhosts:
             self._enable_ocsp_stapling_single(vhost, chain_path)
 
-    def _enable_ocsp_stapling_single(self, vhost, chain_path):
+    def _enable_ocsp_stapling_single(self, vhost: obj.VirtualHost,
+                                     chain_path: Optional[str]) -> None:
         """Include OCSP response in TLS handshake
 
         :param str vhost: vhost to enable OCSP response for
@@ -958,7 +984,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
     ######################################
     # Nginx server management (Installer)
     ######################################
-    def restart(self):
+    def restart(self) -> None:
         """Restarts nginx server.
 
         :raises .errors.MisconfigurationError: If either the reload fails.
@@ -966,7 +992,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         """
         nginx_restart(self.conf('ctl'), self.nginx_conf, self.conf('sleep-seconds'))
 
-    def config_test(self):
+    def config_test(self) -> None:
         """Check the configuration of Nginx for errors.
 
         :raises .errors.MisconfigurationError: If config_test fails
@@ -977,7 +1003,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         except errors.SubprocessError as err:
             raise errors.MisconfigurationError(str(err))
 
-    def _nginx_version(self):
+    def _nginx_version(self) -> str:
         """Return results of nginx -V
 
         :returns: version text
@@ -1001,7 +1027,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
                 "Unable to run %s -V" % self.conf('ctl'))
         return text
 
-    def get_version(self):
+    def get_version(self) -> Tuple[int, ...]:
         """Return version of Nginx Server.
 
         Version is returned as tuple. (ie. 2.4.7 = (2, 4, 7))
@@ -1034,8 +1060,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         product_name, product_version = version_matches[0]
         if product_name != 'nginx':
-            logger.warning("NGINX derivative %s is not officially supported by"
-                           " certbot", product_name)
+            logger.warning("nginx derivative %s is not officially supported by "
+                           "Certbot.", product_name)
 
         nginx_version = tuple(int(i) for i in product_version.split("."))
 
@@ -1046,7 +1072,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         return nginx_version
 
-    def _get_openssl_version(self):
+    def _get_openssl_version(self) -> str:
         """Return version of OpenSSL linked to Nginx.
 
         Version is returned as string. If no version can be found, empty string is returned.
@@ -1068,7 +1094,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
                 return ""
         return matches[0]
 
-    def more_info(self):
+    def more_info(self) -> str:
         """Human-readable string to help understand the module"""
         return (
             "Configures Nginx to authenticate and install HTTPS.{0}"
@@ -1078,7 +1104,8 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
                 version=".".join(str(i) for i in self.version))
         )
 
-    def auth_hint(self, failed_achalls): # pragma: no cover
+    def auth_hint(self,  # pragma: no cover
+                  failed_achalls: Iterable[achallenges.AnnotatedChallenge]) -> str:
         return (
             "The Certificate Authority failed to verify the temporary nginx configuration changes "
             "made by Certbot. Ensure the listed domains point to this nginx server and that it is "
@@ -1088,7 +1115,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
     ###################################################
     # Wrapper functions for Reverter class (Installer)
     ###################################################
-    def save(self, title=None, temporary=False):
+    def save(self, title: Optional[str] = None, temporary: bool = False) -> None:
         """Saves all changes to the configuration files.
 
         :param str title: The title of the save. If a title is given, the
@@ -1112,7 +1139,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         if title and not temporary:
             self.finalize_checkpoint(title)
 
-    def recovery_routine(self):
+    def recovery_routine(self) -> None:
         """Revert all previously modified files.
 
         Reverts all modified files that have not been saved as a checkpoint
@@ -1124,7 +1151,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         self.new_vhost = None
         self.parser.load()
 
-    def revert_challenge_config(self):
+    def revert_challenge_config(self) -> None:
         """Used to cleanup challenge configurations.
 
         :raises .errors.PluginError: If unable to revert the challenge config.
@@ -1134,7 +1161,7 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         self.new_vhost = None
         self.parser.load()
 
-    def rollback_checkpoints(self, rollback=1):
+    def rollback_checkpoints(self, rollback: int = 1) -> None:
         """Rollback saved checkpoints.
 
         :param int rollback: Number of checkpoints to revert
@@ -1150,12 +1177,13 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
     ###########################################################################
     # Challenges Section for Authenticator
     ###########################################################################
-    def get_chall_pref(self, unused_domain):
+    def get_chall_pref(self, unused_domain: str) -> List[Type[challenges.Challenge]]:
         """Return list of challenge preferences."""
         return [challenges.HTTP01]
 
     # Entry point in main.py for performing challenges
-    def perform(self, achalls):
+    def perform(self, achalls: List[achallenges.AnnotatedChallenge]
+                ) -> List[challenges.ChallengeResponse]:
         """Perform the configuration related challenge.
 
         This function currently assumes all challenges will be fulfilled.
@@ -1164,13 +1192,16 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
 
         """
         self._chall_out += len(achalls)
-        responses = [None] * len(achalls)
+        responses: List[Optional[challenges.ChallengeResponse]] = [None] * len(achalls)
         http_doer = http_01.NginxHttp01(self)
 
         for i, achall in enumerate(achalls):
             # Currently also have chall_doer hold associated index of the
             # challenge. This helps to put all of the responses back together
             # when they are all complete.
+            if not isinstance(achall, achallenges.KeyAuthorizationAnnotatedChallenge):
+                raise errors.Error("Challenge should be an instance "
+                                   "of KeyAuthorizationAnnotatedChallenge")
             http_doer.add_chall(achall, i)
 
         http_response = http_doer.perform()
@@ -1184,10 +1215,10 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
         for i, resp in enumerate(http_response):
             responses[http_doer.indices[i]] = resp
 
-        return responses
+        return [response for response in responses if response]
 
     # called after challenges are performed
-    def cleanup(self, achalls):
+    def cleanup(self, achalls: List[achallenges.AnnotatedChallenge]) -> None:
         """Revert all challenges."""
         self._chall_out -= len(achalls)
 
@@ -1197,13 +1228,13 @@ class NginxConfigurator(common.Installer, interfaces.Authenticator):
             self.restart()
 
 
-def _test_block_from_block(block):
+def _test_block_from_block(block: List[Any]) -> List[Any]:
     test_block = nginxparser.UnspacedList(block)
     parser.comment_directive(test_block, 0)
     return test_block[:-1]
 
 
-def _redirect_block_for_domain(domain):
+def _redirect_block_for_domain(domain: str) -> List[Any]:
     updated_domain = domain
     match_symbol = '='
     if util.is_wildcard_domain(domain):
@@ -1219,7 +1250,7 @@ def _redirect_block_for_domain(domain):
     return redirect_block
 
 
-def nginx_restart(nginx_ctl, nginx_conf, sleep_duration):
+def nginx_restart(nginx_ctl: str, nginx_conf: str, sleep_duration: int) -> None:
     """Restarts the Nginx Server.
 
     .. todo:: Nginx restart is fatal if the configuration references
@@ -1264,10 +1295,10 @@ def nginx_restart(nginx_ctl, nginx_conf, sleep_duration):
         time.sleep(sleep_duration)
 
 
-def _determine_default_server_root():
+def _determine_default_server_root() -> str:
     if os.environ.get("CERTBOT_DOCS") == "1":
-        default_server_root = "%s or %s" % (constants.LINUX_SERVER_ROOT,
-            constants.FREEBSD_DARWIN_SERVER_ROOT)
+        default_server_root = (f"{constants.LINUX_SERVER_ROOT} "
+                               f"or {constants.FREEBSD_DARWIN_SERVER_ROOT}")
     else:
         default_server_root = constants.CLI_DEFAULTS["server_root"]
     return default_server_root
