@@ -4,6 +4,7 @@ import re
 import shutil
 import unittest
 from typing import List
+from unittest import mock
 
 from certbot import errors
 from certbot.compat import os
@@ -203,8 +204,7 @@ class NginxParserTest(util.NginxTest):
         mock_vhost.raw = [['listen', '80'],
                           ['ssl', 'on'],
                           ['server_name', '*.www.foo.com', '*.www.example.com']]
-        self.assertTrue(nparser.has_ssl_on_directive(mock_vhost))
-
+        self.assertIs(nparser.has_ssl_on_directive(mock_vhost), True)
 
     def test_remove_server_directives(self):
         nparser = parser.NginxParser(self.config_path)
@@ -452,7 +452,7 @@ class NginxParserTest(util.NginxTest):
         nparser.filedump(ext='')
 
         # check properties of new vhost
-        self.assertFalse(next(iter(new_vhost.addrs)).default)
+        self.assertIs(next(iter(new_vhost.addrs)).default, False)
         self.assertNotEqual(new_vhost.path, default.path)
 
         # check that things are written to file correctly
@@ -461,7 +461,7 @@ class NginxParserTest(util.NginxTest):
         new_defaults = [x for x in new_vhosts if 'default' in x.filep]
         self.assertEqual(len(new_defaults), 2)
         new_vhost_parsed = new_defaults[1]
-        self.assertFalse(next(iter(new_vhost_parsed.addrs)).default)
+        self.assertIs(next(iter(new_vhost_parsed.addrs)).default, False)
         self.assertEqual(next(iter(default.names)), next(iter(new_vhost_parsed.names)))
         self.assertEqual(len(default.raw), len(new_vhost_parsed.raw))
         self.assertTrue(next(iter(default.addrs)).super_eq(next(iter(new_vhost_parsed.addrs))))
@@ -533,5 +533,14 @@ class NginxParserTest(util.NginxTest):
             for output in log.output
         ))
 
+    @mock.patch('certbot_nginx._internal.parser.logger.warning')
+    def test_load_unsupported_directive_logged(self, mock_warn):
+        nparser = parser.NginxParser(self.config_path)
+        nparser.config_root = nparser.abs_path('unsupported_directives.conf')
+        nparser.load()
+        self.assertEqual(mock_warn.call_count, 1)
+        self.assertIn("which is not supported by Certbot", mock_warn.call_args[0][0])
+
+
 if __name__ == "__main__":
     unittest.main()  # pragma: no cover
