@@ -4,10 +4,7 @@ import shutil
 import tempfile
 
 import josepy as jose
-try:
-    import mock
-except ImportError:  # pragma: no cover
-    from unittest import mock  # type: ignore
+from unittest import mock
 import pkg_resources
 
 from certbot import util
